﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using Infragistics.Controls.Menus;
using Infragistics.DragDrop;
using System.Reflection;

namespace DataTreeFiltering
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        ICollectionView view;

        #region Methods and Handlers

        public MainWindow()
        {
            InitializeComponent();

            Data = new ContinentCollection();

            Operators = new ObservableCollection<Operator>()
            {
                new Operator { ID = 1, Name = "Contains" },
                new Operator { ID = 2, Name = "Starts With" },
                new Operator { ID = 3, Name = "Ends With" },
            };
            SelectedOperator = Operators[0];

            FilterModes = new ObservableCollection<Operator>()
            {
                new Operator { ID = 1, Name = "Root" },
                new Operator { ID = 2, Name = "Selected" },
            };
            FilterMode = FilterModes[0];

            DataContext = this;
        }

        private void xamDataTree1_Loaded(object sender, RoutedEventArgs e)
        {
            view = CollectionViewSource.GetDefaultView(xamDataTree1.ItemsSource);
            xamDataTree1.ItemsSource = view;
        }

        private void filter_btn_Click(object sender, RoutedEventArgs e)
        {
            Filter(SelectedOperator, FilterMode);
        }

        private void clearr_btn_Click(object sender, RoutedEventArgs e)
        {
            view.Filter = null;
            foreach (XamDataTreeNode item in xamDataTree1.Nodes)
            {
                ClearFilter(item);
            }
        }

        private void Filter(Operator op, Operator mode)
        {
            if (mode.ID == 1)
            {
                if (op.ID == 1)
                {
                    view.Filter = new Predicate<object>(Contains);
                }
                if (op.ID == 2)
                {
                    view.Filter = new Predicate<object>(StartsWith);
                }
                if (op.ID == 3)
                {
                    view.Filter = new Predicate<object>(EndsWith);
                }
            }

            if (mode.ID == 2)
            {
                if (xamDataTree1.ActiveNode != null)
                {
                    XamDataTreeNode node = xamDataTree1.ActiveNode;

                    if (node.HasChildren)
                    {
                        if (op.ID == 1)
                        {
                            foreach (XamDataTreeNode cNode in node.Nodes)
                            {
                                if (!Contains(cNode.Data))
                                {
                                    FilterNode(cNode);
                                }
                            }
                        }
                        if (op.ID == 2)
                        {
                            foreach (XamDataTreeNode cNode in node.Nodes)
                            {
                                if (!StartsWith(cNode.Data))
                                {
                                    FilterNode(cNode);
                                }
                            }
                        }
                        if (op.ID == 3)
                        {
                            foreach (XamDataTreeNode cNode in node.Nodes)
                            {
                                if (!EndsWith(cNode.Data))
                                {
                                    FilterNode(cNode);
                                }
                            }
                        }
                    }
                }
            }
        }

        private void ClearFilter(XamDataTreeNode t)
        {
            if (t.Control != null)
            {
                t.Control.Tag = null;
                if (t.HasChildren)
                {
                    foreach (XamDataTreeNode node in t.Nodes)
                    {
                        ClearFilter(node);
                    }
                }
                
            }
        }

        private void FilterNode(XamDataTreeNode t)
        {
            if (t.Control != null)
            {
                t.Control.Tag = "Filtered";
                if (t.HasChildren)
                {
                    foreach (XamDataTreeNode node in t.Nodes)
                    {
                        FilterNode(node);
                    }
                }

            }
        }

        #endregion

        #region Filter Methods
        public bool Contains(object param)
        {
            if (param is Continent)
            {
                ICollectionView countriesView = CollectionViewSource.GetDefaultView((param as Continent).Children);
                countriesView.Filter = new Predicate<object>(Contains);

                if (countriesView.IsEmpty)
                    return (param as Continent).Name.ToLower().Contains(FilterString.ToLower());
                else
                    return true;

                //
            }
            if (param is Country)
            {
                ICollectionView citiesView = CollectionViewSource.GetDefaultView((param as Country).Children);
                citiesView.Filter = new Predicate<object>(Contains);

                if (citiesView.IsEmpty)
                    return (param as Country).Name.ToLower().Contains(FilterString.ToLower());
                else
                    return true;
            }
            if (param is City)
            {
                return (param as City).Name.ToLower().Contains(FilterString.ToLower());
            }
            return true;
        }

        public bool StartsWith(object param)
        {
            if (param is Continent)
            {
                return (param as Continent).Name.ToLower().StartsWith(FilterString.ToLower());
            }
            if (param is Country)
            {
                return (param as Country).Name.ToLower().StartsWith(FilterString.ToLower());
            }
            if (param is City)
            {
                return (param as City).Name.ToLower().StartsWith(FilterString.ToLower());
            }
            return true;
        }

        public bool EndsWith(object param)
        {
            if (param is Continent)
            {
                return (param as Continent).Name.ToLower().EndsWith(FilterString.ToLower());
            }
            if (param is Country)
            {
                return (param as Country).Name.ToLower().EndsWith(FilterString.ToLower());
            }
            if (param is City)
            {
                return (param as City).Name.ToLower().EndsWith(FilterString.ToLower());
            }
            return true;
        }
        #endregion

        #region Proeprties

        public string FilterString { get; set; }

        public Operator SelectedOperator { get; set; }

        public Operator FilterMode { get; set; }
        
        public ObservableCollection<Continent> Data { get; set; }

        public ObservableCollection<Operator> Operators { get; set; }

        public ObservableCollection<Operator> FilterModes { get; set; }

        #endregion  

    }

    #region Data

    public class ContinentCollection : ObservableCollection<Continent>
    {
        public ContinentCollection()
        {
            Add(new Continent
            {
                Name = "Africa",
                Children = new ObservableCollection<Country>()
                {
                    new Country()
                    {
                        Name = "Algeria",
                        Children = new ObservableCollection<City>()
                        {
                            new City {Name = "Algiers"},
                            new City {Name = "Oran"},
                        }
                    },
                    new Country()
                    {
                        Name = "Angola",
                        Children = new ObservableCollection<City>()
                        {
                            new City {Name = "Soyo"},
                            new City {Name = "Ndalatando"},
                            new City {Name = "Ambriz"},
                        }
                    },
                    new Country()
                    {
                        Name = "Cape Verde",
                        Children = new ObservableCollection<City>()
                        {
                            new City {Name = "Mindelo"},
                            new City {Name = "Assomada"},
                        }
                    }
                },
            });

            Add(new Continent
            {
                Name = "Asia",
                Children = new ObservableCollection<Country>()
                {
                    new Country()
                    {
                        Name = "Cambodia",
                        Children = new ObservableCollection<City>()
                        {
                            new City {Name = "Phnom Penh"},
                            new City {Name = "Koh Kong"},
                        }
                    },
                    new Country()
                    {
                        Name = "China",
                        Children = new ObservableCollection<City>()
                        {
                            new City {Name = "Shanghai"},
                            new City {Name = "Beijing"},
                            new City {Name = "Tianjin"},
                        }
                    },
                    new Country()
                    {
                        Name = "Japan",
                        Children = new ObservableCollection<City>()
                        {
                            new City {Name = "Tokyo"},
                            new City {Name = "Yokohama"},
                            new City {Name = "Sapporo"},
                        }
                    }
                }
            });

            Add(new Continent
            {
                Name = "Australia",
                Children = new ObservableCollection<Country>()
                {
                    new Country()
                    {
                        Name = "Australia",
                        Children = new ObservableCollection<City>()
                        {
                            new City {Name = "Sydney"},
                            new City {Name = "Melbourne"},
                            new City {Name = "Perth"},
                        }
                    }
                }
            });

            Add(new Continent
            {
                Name = "Europe",
                Children = new ObservableCollection<Country>()
                {
                    new Country()
                    {
                        Name = "Germany",
                        Children = new ObservableCollection<City>()
                        {
                            new City {Name = "Berlin"},
                            new City {Name = "Hamburg"}
                        }
                    },
                    new Country()
                    {
                        Name = "Italy",
                        Children = new ObservableCollection<City>()
                        {
                            new City {Name = "Milan"},
                            new City {Name = "Naples"},
                            new City {Name = "Rome"}
                        }
                    }
                },
            });
        }
    }

    public class Continent
    {
        public string Name { get; set; }
        public ObservableCollection<Country> Children { get; set; }
        public ICollectionView CountresView { get; set; }
    }

    public class Country
    {
        public string Name { get; set; }
        public ObservableCollection<City> Children { get; set; }
    }

    public class City
    {
        public string Name { get; set; }
    }

    public class Operator
    {
        public int ID { get; set; }
        public string Name { get; set; }
    }

    #endregion 
}
